from chroma_feedback import helper
from chroma_feedback.typing import Producer, Status


def normalize_data(slug : str, url : str, status : str, conclusion : str) -> Producer:
	return\
	{
		'name': 'github',
		'slug': slug,
		'url': url,
		'status': normalize_status(status, conclusion)
	}


def normalize_status(status : str, conclusion : str) -> Status:
	status = helper.to_lower_case(status)
	conclusion = helper.to_lower_case(conclusion)

	if conclusion == 'skipped':
		return 'skipped'
	if status in ['in_progress', 'queued']:
		return 'started'
	if conclusion in ['cancelled', 'stale']:
		return 'errored'
	if conclusion in ['action_required', 'timed_out']:
		return 'warned'
	if conclusion == 'failure':
		return 'failed'
	return 'passed'



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--teamcity-host', default = 'https://teamcity.jetbrains.com')
		program.add_argument('--teamcity-slug', action = 'append')
		program.add_argument('--teamcity-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.teamcity_slug:
		for slug in ARGS.teamcity_slug:
			result.extend(fetch(ARGS.teamcity_host, slug, ARGS.teamcity_token))
	else:
		result.extend(fetch(ARGS.teamcity_host, None, ARGS.teamcity_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/app/rest/buildTypes/?fields=buildType(builds($locator(running:any),build(webUrl,status,running,buildType(projectName,paused))))&locator=affectedProject:(id:' + slug + ')', headers = request.create_bearer_auth_headers(token))
	elif host and token:
		response = request.get(host + '/app/rest/buildTypes/?fields=buildType(builds($locator(running:any),build(webUrl,status,running,buildType(projectName,paused))))', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'buildType' in data:
			for project in data['buildType']:
				if 'builds' in project and 'build' in project['builds']:
					build = helper.get_first(project['builds']['build'])

					if build and 'buildType' in build and 'projectName' in build['buildType'] and 'paused' in build['buildType'] and 'webUrl' in build and 'status' in build and 'running' in build:
						result.append(normalize_data(build['buildType']['projectName'], build['webUrl'], build['status'], build['buildType']['paused'], build['running']))
	return result



from logging import basicConfig, DEBUG, ERROR, getLogger, INFO, Logger, WARNING
from typing import Any

from chroma_feedback.typing import LogLevel


def init(log_level : LogLevel) -> None:
	basicConfig(format = '')
	get_package_logger().setLevel(get_log_level()[log_level])


def get_package_logger() -> Logger:
	return getLogger('chroma_feedback')


def debug(message : str = '') -> None:
	get_package_logger().debug(message)


def info(message : str = '') -> None:
	get_package_logger().info(message)


def warn(message : str = '') -> None:
	get_package_logger().warning(message)


def error(message : str = '') -> None:
	get_package_logger().error(message)


def get_log_level() -> Any:
	return\
	{
		'error': ERROR,
		'warn': WARNING,
		'info': INFO,
		'debug': DEBUG
	}



import importlib
import sys
from argparse import ArgumentParser
from typing import Any, List

from chroma_feedback import helper, logger, wording
from chroma_feedback.typing import Consumer, ProducerReport


def process(program : ArgumentParser, producer_report : List[ProducerReport]) -> List[Consumer]:
	args = helper.get_first(program.parse_known_args())
	result = []

	for consumer_name in args.consumer:
		consumer = load_consumer(consumer_name)

		if consumer.support() is True:
			try:
				consumer.init(program)
				result.extend(consumer.run(producer_report))
			except:
				logger.error(wording.get('consumer_crashed').format(consumer_name) + wording.get('exclamation_mark'))
				sys.exit()
		else:
			logger.error(wording.get('consumer_not_supported').format(consumer_name) + wording.get('exclamation_mark'))
			sys.exit()
	return result


def load_consumer(consumer_name : str) -> Any:
	try:
		return importlib.import_module('chroma_feedback.consumer.' + consumer_name)
	except ImportError:
		logger.error(wording.get('consumer_not_found').format(consumer_name) + wording.get('exclamation_mark'))
		sys.exit()



WORDING =\
{
	'version_not_supported': 'Python {}.{} not supported',
	'package_not_found': 'Package {} not found',
	'connection_not_found': 'Connection to {} not found',
	'daemon_not_found': 'Daemon for {} not found',
	'ip_not_found': 'IP for {} not found',
	'device_not_found': 'Device not found',
	'light_not_found': 'Light not found',
	'group_not_found': 'Group not found',
	'producer_not_found': 'Producer {} not found',
	'producer_crashed': 'Producer {} crashed',
	'consumer_not_supported': 'Consumer {} not supported',
	'consumer_not_found': 'Consumer {} not found',
	'consumer_crashed': 'Consumer {} crashed',
	'result_not_found': 'Result not found',
	'fetch_request': 'Fetch {} {} {}',
	'retry_request': 'Retry {} request in {} seconds',
	'has_status': 'Status of {} on {} is {}',
	'set_status': 'Set status of {} to {}',
	'press_button': 'Press {} button on {}',
	'enable_feature': 'Enable {} on {}',
	'goodbye': 'Goodbye',
	'start': 'Start',
	'stop': 'Stop',
	'exit': 'Exit',
	'by': 'by',
	'hourglass': '\u29d7',
	'tick': '\u2714',
	'cross': '\u2716',
	'point': '.',
	'comma': ',',
	'colon': ':',
	'question_mark': '?',
	'exclamation_mark': '!'
}


def get(key : str) -> str:
	return WORDING[key]



from chroma_feedback import helper
from chroma_feedback.typing import Producer, Status


def normalize_data(slug : str, status : str) -> Producer:
	return\
	{
		'name': 'heroku',
		'slug': slug,
		'url': None,
		'status': normalize_status(status)
	}


def normalize_status(status : str) -> Status:
	status = helper.to_lower_case(status)

	if status == 'pending':
		return 'started'
	if status == 'failed':
		return 'failed'
	return 'passed'



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--custom-host', default = 'http://localhost')
		program.add_argument('--custom-slug', action = 'append', required = True)
		program.add_argument('--custom-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.custom_slug:
		result.extend(fetch(ARGS.custom_host, slug, ARGS.custom_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/statuses/' + slug, headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		for build in data:
			if 'slug' in build and 'url' in build and 'status' in build:
				result.append(normalize_data(build['slug'], build['url'], build['status']))
	return result



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--appveyor-host', default = 'https://ci.appveyor.com')
		program.add_argument('--appveyor-slug', action = 'append')
		program.add_argument('--appveyor-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.appveyor_slug:
		for slug in ARGS.appveyor_slug:
			result.extend(fetch(ARGS.appveyor_host, slug, ARGS.appveyor_token))
	else:
		result.extend(fetch(ARGS.appveyor_host, None, ARGS.appveyor_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/api/projects/' + slug, headers = request.create_bearer_auth_headers(token))
	elif host and token:
		response = request.get(host + '/api/projects', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'project' in data and 'accountName' in data['project'] and 'build' in data and 'status' in data['build']:
			result.append(normalize_data(data['project']['accountName'] + '/' + data['project']['slug'], data['build']['status']))
		if 'builds' in helper.get_first(data):
			for project in data:
				build = helper.get_first(project['builds'])

				if project and 'accountName' in project and 'slug' in project and build and 'status' in build:
					result.append(normalize_data(project['accountName'] + '/' + project['slug'], build['status']))
	return result



from .core import process

__all__ =\
[
	'appveyor',
	'azure',
	'bamboo',
	'bitbucket',
	'buddy',
	'circle',
	'codeship',
	'custom',
	'datadog',
	'github',
	'gitlab',
	'heroku',
	'jenkins',
	'netlify',
	'teamcity',
	'travis',
	'vercel'
]



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--bitbucket-host', default = 'https://api.bitbucket.org')
		program.add_argument('--bitbucket-slug', action = 'append', required = True)
		program.add_argument('--bitbucket-username', required = True)
		program.add_argument('--bitbucket-password', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.bitbucket_slug:
		result.extend(fetch(ARGS.bitbucket_host, slug, ARGS.bitbucket_username, ARGS.bitbucket_password))
	return result


def fetch(host : str, slug : str, username : str, password : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and username and password:
		response = request.get(host + '/2.0/repositories/' + slug + '/pipelines/', headers = request.create_basic_auth_headers(username, password))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'values' in data:
			build = helper.get_last(data['values'])

			if build and 'repository' in build and 'full_name' in build['repository'] and 'state' in build and 'name' in build['state']:
				if 'result' in build['state'] and 'name' in build['state']['result']:
					result.append(normalize_data(build['repository']['full_name'], build['state']['name'], build['state']['result']['name']))
				else:
					result.append(normalize_data(build['repository']['full_name'], build['state']['name'], None))
	return result



import sys
from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, logger, wording
from chroma_feedback.typing import Consumer, ProducerReport
from .group import get_groups, process_groups
from .light import filter_lights, get_lights, process_lights

ARGS = None


def support() -> bool:
	return helper.is_linux() is True or helper.is_mac() is True or helper.is_windows() is True


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--lifx-light-group-name', action = 'append')
		program.add_argument('--lifx-light-light-ip', action = 'append')
	ARGS = helper.get_first(program.parse_known_args())


def run(producer_report : List[ProducerReport]) -> List[Consumer]:
	if ARGS.lifx_light_group_name:
		groups = get_groups(ARGS.lifx_light_group_name)

		if not groups:
			logger.error(wording.get('group_not_found') + wording.get('exclamation_mark'))
			sys.exit()
		return process_groups(groups, producer_report)

	# fallback as needed

	lights = filter_lights(get_lights(), ARGS.lifx_light_light_ip)

	if not lights:
		logger.error(wording.get('light_not_found') + wording.get('exclamation_mark'))
		sys.exit()
	return process_lights(lights, producer_report)



from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

LIGHTS : List[Any] = []


def get_lights(light_ips : List[str]) -> List[Any]:
	global LIGHTS

	if not LIGHTS:
		for light_ip in light_ips:
			LIGHTS.append(get_api(light_ip))
	return LIGHTS


def process_lights(lights : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process lights

	for light in lights:
		if set_light(light, color.get_by_status(status)):
			result.append(
			{
				'name': 'xiaomi_yeelight',
				'type': 'light',
				'description': helper.create_description(light.get_properties()['name'], light._ip),
				'status': status
			})
	return result


def set_light(light : Any, color_config : Color) -> bool:
	return light.turn_on() == 'ok' and light.set_rgb(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2]) == 'ok'



import sys
from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, logger, wording
from chroma_feedback.typing import Consumer, ProducerReport
from .light import filter_lights, get_lights, process_lights

ARGS = None


def support() -> bool:
	return helper.is_linux() is True or helper.is_mac() is True


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--luxafor-flag-light-id', action = 'append')
	ARGS = helper.get_first(program.parse_known_args())


def run(producer_report : List[ProducerReport]) -> List[Consumer]:
	light = filter_lights(get_lights(), ARGS.luxafor_flag_light_id)

	if not light:
		logger.error(wording.get('device_not_found') + wording.get('exclamation_mark'))
		sys.exit()
	return process_lights(light, producer_report)



import signal
import sys
from argparse import ArgumentParser

from chroma_feedback import consumer, helper, logger, loop, metadata, producer, reporter, systray, wording

INTERVAL = 0


def cli() -> None:
	signal.signal(signal.SIGINT, lambda signal_number, frame: destroy())
	program = ArgumentParser()
	program.add_argument('-V', '--version', action = 'version', version = metadata.get('name') + ' ' + metadata.get('version'))
	program.add_argument('-P', '--producer', action = 'append', choices = producer.__all__, required = True)
	program.add_argument('-C', '--consumer', action = 'append', choices = consumer.__all__, required = helper.has_argument('--dry-run') is False and helper.has_argument('-D') is False)
	program.add_argument('-I', '--background-interval', default = 60, type = int)
	program.add_argument('-B', '--background-run', action = 'store_true')
	program.add_argument('-D', '--dry-run', action = 'store_true')
	program.add_argument('-L', '--log-level', default = 'info', choices = logger.get_log_level())
	init(program)


def init(program : ArgumentParser) -> None:
	args = helper.get_first(program.parse_known_args())

	# init logger

	logger.init(args.log_level)

	# validate version

	if sys.version_info < (3, 8):
		logger.error(wording.get('version_not_supported').format(sys.version_info.major, sys.version_info.minor) + wording.get('exclamation_mark'))
		sys.exit()

	# report header

	reporter.print_header()

	# handle loop

	application = loop.get_application()
	timer = loop.get_timer()
	timer.setInterval(100)
	if args.background_run is True:
		timer.timeout.connect(lambda : background_run(program))
	else:
		timer.timeout.connect(lambda : sys.exit())
	timer.singleShot(0, lambda : run(program))
	timer.start()
	sys.exit(application.exec_())


def background_run(program : ArgumentParser) -> None:
	global INTERVAL

	args = helper.get_first(program.parse_known_args())
	timer = loop.get_timer()

	# handle interval

	if INTERVAL == args.background_interval * 1000:
		run(program)
		INTERVAL = 0
	else:
		INTERVAL += timer.interval()


def run(program : ArgumentParser) -> None:
	args = helper.get_first(program.parse_known_args())

	# process producer

	producer_result = producer.process(program)

	# handle exit

	if not producer_result:
		logger.error(wording.get('result_not_found') + wording.get('exclamation_mark'))
		sys.exit()

	# report producer

	producer_report = reporter.create_producer_report(producer_result)

	if producer_report:
		reporter.print_report(producer_report)

	# handle dry run

	if args.dry_run is False:

		# process consumer

		consumer_result = consumer.process(program, producer_report)

		# report consumer

		consumer_report = reporter.create_consumer_report(consumer_result)

		if consumer_report:
			reporter.print_report(consumer_report)

	# handle systray

	if args.background_run is True and loop.is_created() is True:
		if systray.is_created() is True:
			systray.update(producer_report)
		else:
			systray.create(producer_report)


def destroy() -> None:
	logger.info(wording.get('goodbye') + wording.get('exclamation_mark'))
	sys.exit()



from typing import Dict, List, Literal, Optional, TypedDict

Status = Literal\
[
	'skipped',
	'passed',
	'started',
	'errored',
	'warned',
	'failed'
]
ProducerName = Literal\
[
	'appveyor',
	'azure',
	'bamboo',
	'bitbucket',
	'buddy',
	'circle',
	'codeship',
	'custom',
	'datadog',
	'github',
	'gitlab',
	'heroku',
	'jenkins',
	'netlify',
	'teamcity',
	'travis',
	'vercel'
]
ConsumerName = Literal\
[
	'agile_innovative_blinkstick',
	'elgato_streamdeck',
	'embrava_blynclight',
	'kuando_busylight',
	'lifx_light',
	'luxafor_flag',
	'magic_hue',
	'nanoleaf_light',
	'philips_hue',
	'razer_chroma',
	'thingm_blink1',
	'wiz_light',
	'xiaomi_yeelight'
]
ConsumerType = Literal\
[
	'light',
	'device',
	'group'
]
LogLevel = Literal\
[
	'error',
	'warn',
	'info',
	'debug'
]
Headers = Dict[str, str]

class Producer(TypedDict):
	name: ProducerName
	slug: str
	url: Optional[str]
	status: Status


class Consumer(TypedDict):
	name: ConsumerName
	type: ConsumerType
	description: str
	status: Status


class ProducerReport(TypedDict):
	name: ProducerName
	symbol: str
	message : str
	url : Optional[str]
	status : Status


class ConsumerReport(TypedDict):
	name: ConsumerName
	symbol: str
	message : str
	status : Status


class Color(TypedDict):
	name: str
	rgb: List[int]
	hue: int
	saturation: List[int]
	brightness: List[int]
	kelvin: int



import sys
from typing import Any

from chroma_feedback import logger, wording

API = None


def get_api(light_ip : str) -> Any:
	global API

	if not API:
		API = api_factory(light_ip)
	return API


def api_factory(light_ip : str) -> Any:
	try:
		from magichue import Light

		try:
			api = Light(light_ip)
		except OSError:
			logger.error(wording.get('connection_not_found').format('magic_hue') + wording.get('exclamation_mark'))
			sys.exit()
		return api
	except ImportError:
		logger.error(wording.get('package_not_found').format('python-magichue') + wording.get('exclamation_mark'))
		sys.exit()


def get_modes() -> Any:
	try:
		from magichue import modes

		return modes
	except ImportError:
		logger.error(wording.get('package_not_found').format('magic_hue') + wording.get('exclamation_mark'))
		sys.exit()



import sys
from typing import Any

from chroma_feedback import logger, wording

API = None


def get_api(bridge_ip : str) -> Any:
	global API

	if not API:
		API = api_factory(bridge_ip)
	return API


def api_factory(bridge_ip : str) -> Any:
	try:
		from phue import Bridge, PhueRegistrationException, PhueRequestTimeout

		try:
			api = Bridge(bridge_ip)
		except (PhueRequestTimeout, OSError, ValueError):
			logger.error(wording.get('connection_not_found').format('philips_hue') + wording.get('exclamation_mark'))
			sys.exit()
		except PhueRegistrationException:
			logger.error(wording.get('press_button').format('pairing', 'philips_hue') + wording.get('exclamation_mark'))
			sys.exit()
		return api
	except ImportError:
		logger.error(wording.get('package_not_found').format('phue') + wording.get('exclamation_mark'))
		sys.exit()



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Headers, Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--gitlab-host', default = 'https://gitlab.com')
		program.add_argument('--gitlab-slug', action = 'append')
		program.add_argument('--gitlab-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.gitlab_slug:
		for slug in ARGS.gitlab_slug:
			pipeline_ids = fetch_pipeline_ids(ARGS.gitlab_host, slug, ARGS.gitlab_token)

			if pipeline_ids:
				for pipeline_id in pipeline_ids:
					result.extend(fetch(ARGS.gitlab_host, slug, pipeline_id, ARGS.gitlab_token))
	else:
		project_ids = fetch_project_ids(ARGS.gitlab_host, ARGS.gitlab_token)

		for project_id in project_ids:
			pipeline_ids = fetch_pipeline_ids(ARGS.gitlab_host, project_id, ARGS.gitlab_token)

			if pipeline_ids:
			 	for pipeline_id in pipeline_ids:
			 		result.extend(fetch(ARGS.gitlab_host, project_id, pipeline_id, ARGS.gitlab_token))
	return result


def fetch(host : str, slug : str, pipeline_id : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and pipeline_id and token:
		response = request.get(host + '/api/v4/projects/' + slug + '/pipelines/' + pipeline_id + '/jobs', headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		for build in data:
			if 'name' in build and 'web_url' in build and 'status' in build:
				result.append(normalize_data(slug + '/' + build['name'], build['web_url'], build['status']))
	return result


def fetch_project_ids(host : str, token : str) -> List[str]:
	result = []
	response = None

	if host and  token:
		response = request.get(host + '/api/v4/projects?owned=true', headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		for project in data:
			if 'id' in project:
				result.append(str(project['id']))
	return result


def fetch_pipeline_ids(host : str, slug : str, token : str) -> List[str]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/api/v4/projects/' + slug + '/pipelines', headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)
		pipeline = helper.get_first(data)

		if pipeline and 'id' in pipeline:
			result.append(str(pipeline['id']))
	return result


def create_headers(token : str) -> Headers:
	return\
	{
		'Accept': 'application/json',
		'Private-Token': token
	}



from chroma_feedback import helper
from chroma_feedback.typing import Producer, Status


def normalize_data(slug : str, result : str, is_building : bool) -> Producer:
	return\
	{
		'name': 'jenkins',
		'slug': slug,
		'url': None,
		'status': normalize_status(result, is_building)
	}


def normalize_status(result : str, is_building: bool) -> Status:
	result = helper.to_lower_case(result)

	if is_building is True:
		return 'started'
	if result in ['unstable', 'not_build']:
		return 'errored'
	if result == 'failure':
		return 'failed'
	return 'passed'



import sys
from typing import Any

from chroma_feedback import logger, wording

API = None


def get_api() -> Any:
	global API

	if not API:
		API = api_factory()
	return API


def api_factory() -> Any:
	try:
		from lifxlan import LifxLAN, WorkflowException

		try:
			api = LifxLAN()
		except WorkflowException:
			logger.error(wording.get('connection_not_found').format('lifx_light') + wording.get('exclamation_mark'))
			sys.exit()
		return api
	except ImportError:
		logger.error(wording.get('package_not_found').format('lifxlan') + wording.get('exclamation_mark'))
		sys.exit()



import importlib
import sys
from argparse import ArgumentParser
from typing import Any, List

from chroma_feedback import helper, logger, wording
from chroma_feedback.typing import Producer


def process(program : ArgumentParser) -> List[Producer]:
	args = helper.get_first(program.parse_known_args())
	result = []

	for producer_name in args.producer:
		producer = load_producer(producer_name)
		try:
			producer.init(program)
			result.extend(producer.run())
		except:
			logger.error(wording.get('producer_crashed').format(producer_name) + wording.get('exclamation_mark'))
			sys.exit()
	return result


def load_producer(producer_name : str) -> Any:
	try:
		return importlib.import_module('chroma_feedback.producer.' + producer_name)
	except ImportError:
		logger.error(wording.get('producer_not_found').format(producer_name) + wording.get('exclamation_mark'))
		sys.exit()



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--netlify-host', default = 'https://api.netlify.com')
		program.add_argument('--netlify-slug', action = 'append')
		program.add_argument('--netlify-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.netlify_slug:
		for slug in ARGS.netlify_slug:
			result.extend(fetch(ARGS.netlify_host, slug, ARGS.netlify_token))
	else:
		site_ids = fetch_site_ids(ARGS.netlify_host, ARGS.netlify_token)

		if site_ids:
			for site_id in site_ids:
				result.extend(fetch(ARGS.netlify_host, site_id, ARGS.netlify_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result : List[Producer] = []
	response = None

	if host and slug and token:
		response = request.get(host + '/api/v1/sites/' + slug + '/deploys', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)
		build = helper.get_first(data)

		if build and 'name' in build and 'admin_url' in build and 'state' in build:
			result.append(normalize_data(build['name'], build['admin_url'], build['state']))
	return result


def fetch_site_ids(host : str, token : str) -> List[str]:
	result = []
	response = None

	if host and token:
		response = request.get(host + '/api/v1/sites', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		for site in data:
			if 'id' in site:
				result.append(site['id'])
	return result



import copy
from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

LIGHTS = None


def get_lights() -> Any:
	global LIGHTS

	if not LIGHTS:
		LIGHTS = get_api().all_lights()
	return LIGHTS


def filter_lights(lights : Any, light_ids : List[str]) -> Any:
	if light_ids:
		for light in copy.copy(lights):
			if light.path not in light_ids:
				lights.remove(light)
	return lights


def process_lights(lights : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process lights

	for light in lights:
		if set_light(light, color.get_by_status(status)):
			result.append(
			{
				'name': 'luxafor_flag',
				'type': 'light',
				'description': helper.create_description(light.info['product_string'], light.path),
				'status': status
			})
	return result


def set_light(light : Any, color_config : Color) -> bool:
	light.on(tuple(color_config['rgb']))
	return light.is_on is True




from argparse import ArgumentParser
from typing import Any, List, Optional

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--vercel-host', default = 'https://api.vercel.com')
		program.add_argument('--vercel-slug', action = 'append')
		program.add_argument('--vercel-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.vercel_slug:
		for slug in ARGS.vercel_slug:
			result.extend(fetch(ARGS.vercel_host, slug, ARGS.vercel_token))
	else:
		result.extend(fetch(ARGS.vercel_host, None, ARGS.vercel_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result : List[Producer] = []
	response = None

	if host and slug and token:
		response = request.get(host + '/v1/projects/' + slug, headers = request.create_bearer_auth_headers(token))
	elif host and token:
		response = request.get(host + '/v1/projects', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'name' in data and 'latestDeployments' in data:
			result.append(_normalize_data(data))
		else:
			for project in data:
				if 'name' in project and 'latestDeployments' in project:
					result.append(_normalize_data(project))
	return result


def _normalize_data(project : Any) -> Optional[Producer]:
	deployment = helper.get_first(project['latestDeployments'])

	if 'readyState' in deployment:
		return normalize_data(project['name'], deployment['readyState'])
	return None



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--azure-host', default = 'https://dev.azure.com')
		program.add_argument('--azure-slug', action = 'append', required = True)
		program.add_argument('--azure-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.azure_slug:
		result.extend(fetch(ARGS.azure_host, slug, ARGS.azure_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/' + slug + '/_apis/build/builds?api-version=6.0', headers = request.create_basic_auth_headers('', token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'value' in data:
			build = helper.get_first(data['value'])

			if build and 'project' in build and 'name' in build['project'] and 'status' in build:
				if 'result' in build:
					result.append(normalize_data(build['project']['name'], build['status'], build['result']))
				else:
					result.append(normalize_data(build['project']['name'], build['status'], None))
	return result



from argparse import ArgumentParser
from typing import Any, List, Optional

from chroma_feedback import helper, request
from chroma_feedback.typing import Headers, Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--travis-host', default = 'https://api.travis-ci.com')
		program.add_argument('--travis-slug', action = 'append', required = True)
		program.add_argument('--travis-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.travis_slug:
		result.extend(fetch(ARGS.travis_host, slug, ARGS.travis_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/repos/' + slug, headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'repo' in data:
			result.append(_normalize_data(data['repo']))
		elif 'repos' in data:
			for repository in data['repos']:
				result.append(_normalize_data(repository))
	return result


def _normalize_data(repository : Any) -> Optional[Producer]:
	if 'slug' in repository and 'last_build_state' in repository and 'active' in repository:
		return normalize_data(repository['slug'], repository['last_build_state'], repository['active'])
	return None


def create_headers(token : str) -> Headers:
	return\
	{
		'Accept': 'application/vnd.travis-ci.2.1+json',
		'Authorization': 'Token ' + token
	}



import sys
from typing import Any

from chroma_feedback import logger, wording

API = None


def get_api(light_ip : str) -> Any:
	global API

	if not API:
		API = api_factory(light_ip)
	return API


def api_factory(light_ip : str) -> Any:
	try:
		from yeelight import Bulb, BulbException

		try:
			api = Bulb(light_ip)
		except BulbException:
			logger.error(wording.get('connection_not_found').format('xiaomi_yeelight') + wording.get('exclamation_mark'))
			sys.exit()
		try:
			api.turn_on()
		except BulbException:
			logger.error(wording.get('enable_feature').format('lan control', 'xiaomi_yeelight') + wording.get('exclamation_mark'))
			sys.exit()
		return api
	except ImportError:
		logger.error(wording.get('package_not_found').format('yeelight') + wording.get('exclamation_mark'))
		sys.exit()



import atexit
import copy
import webbrowser
from typing import Any, List

from PyQt5 import QtCore
from PyQt5.QtCore import QRect, Qt
from PyQt5.QtGui import QColor, QFont, QPainter, QPen, QPixmap, QTransform

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Consumer, ProducerReport, Status
from .api import get_api

DEVICES = None


def get_devices() -> Any:
	global DEVICES

	if not DEVICES:
		DEVICES = get_api().enumerate()
	return DEVICES


def filter_devices(devices : Any, device_ids : List[str]) -> Any:
	if device_ids:
		for device in copy.copy(devices):
			if device.id() not in device_ids:
				devices.remove(device)
	return devices


def process_devices(devices : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process devices

	for device in devices:
		if set_device(device, producer_report):
			result.append(
			{
				'name': 'elgato_streamdeck',
				'type': 'device',
				'description': helper.create_description(device.deck_type(), device.id()),
				'status': status
			})
	return result


def set_device(device : Any, producer_report : List[ProducerReport]) -> bool:
	device.open()

	# process report

	for index, report in enumerate(producer_report):
		if index < device.key_count():
			device.set_key_image(index, create_image(device, report))
			if 'url' in report and report['url']:
				device.set_key_callback(lambda __, key, state : state is True and webbrowser.open(producer_report[key]['url']))

	# smooth reset

	for index in range(len(producer_report), device.key_count()):
		device.set_key_image(index, None)

	# close on destroy

	atexit.register(lambda : device.close())
	return device.is_open() is True


def create_image(device : Any, report : ProducerReport) -> Any:
	color_config = color.get_by_status(report['status'])
	image_config = device.key_image_format()
	transform = create_transform(image_config)
	font_size = image_config['size'][1] / 8
	pixmap = QPixmap(image_config['size'][0], image_config['size'][1])
	pixmap.fill(Qt.transparent)
	painter = QPainter(pixmap)
	painter.setFont(create_font(font_size * 2.75, QFont.Normal))
	painter.setPen(QPen(QColor(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2])))
	painter.drawText(QRect(0, font_size * 1.25, image_config['size'][0], image_config['size'][1]), Qt.AlignCenter, report['symbol'])
	painter.setFont(create_font(font_size * 0.875, QFont.Bold))
	painter.setPen(QPen(Qt.white))
	painter.drawText(QRect(0, font_size * -2, image_config['size'][0], image_config['size'][1]), Qt.AlignCenter, report['name'].upper())
	painter.end()
	return pixmap_to_bytes(pixmap.transformed(transform), image_config)


def create_transform(image_config : Any) -> QTransform:
	transform = QTransform()

	if image_config['flip'][0] is True:
		transform.scale(-1, 1)
	if image_config['flip'][1] is True:
		transform.scale(1, -1)
	if image_config['rotation']:
		transform.rotate(image_config['rotation'])
	return transform


def create_font(font_size : int, font_weight : int) -> QFont:
	font = QFont()
	font.setPointSize(font_size)
	font.setWeight(font_weight)
	return font


def pixmap_to_bytes(pixmap : QPixmap, image_config : Any) -> bytes:
	byte_array = QtCore.QByteArray()
	buffer = QtCore.QBuffer(byte_array)
	buffer.open(QtCore.QIODevice.WriteOnly)
	pixmap.save(buffer, image_config['format'])
	return byte_array.data()



from typing import Any, List

from chroma_feedback import color, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api, get_modes

LIGHTS : List[Any] = []


def get_lights(light_ips : List[str]) -> List[Any]:
	global LIGHTS

	if not LIGHTS:
		for light_ip in light_ips:
			LIGHTS.append(get_api(light_ip))
	return LIGHTS


def process_lights(lights : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process lights

	for light in lights:
		set_light(light, color.get_by_status(status))
		result.append(
		{
			'name': 'magic_hue',
			'type': 'light',
			'description': light.name,
			'status': status
		})
	return result


def set_light(light : Any, color_config : Color) -> None:
	modes = get_modes()

	if modes:
		light.mode = modes.CustomMode(
			mode = modes.MODE_GRADUALLY,
			speed = 1,
			colors =\
			[
				color_config['rgb'],
				color_config['rgb']
			]
		)
	return light.update_status()



from chroma_feedback.typing import Producer, Status


def normalize_data(slug : str, url : str, status : Status) -> Producer:
	return\
	{
		'name': 'custom',
		'slug': slug,
		'url': url,
		'status': status
	}



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Headers, Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--circle-host', default = 'https://circleci.com')
		program.add_argument('--circle-organization')
		program.add_argument('--circle-slug', action = 'append')
		program.add_argument('--circle-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.circle_slug:
		for slug in ARGS.circle_slug:
			pipeline_ids = fetch_pipeline_ids(ARGS.circle_host, None, slug, ARGS.circle_token)

			if pipeline_ids:
				for pipeline_id in pipeline_ids:
					result.extend(fetch(ARGS.circle_host, pipeline_id, ARGS.circle_token))
	elif ARGS.circle_organization:
		pipeline_ids = fetch_pipeline_ids(ARGS.circle_host, ARGS.circle_organization, None, ARGS.circle_token)

		if pipeline_ids:
			for pipeline_id in pipeline_ids:
				result.extend(fetch(ARGS.circle_host, pipeline_id, ARGS.circle_token))
	return result


def fetch(host : str, pipeline_id : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and pipeline_id and token:
		response = request.get(host + '/api/v2/pipeline/' + pipeline_id + '/workflow', headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'items' in data:
			for build in data['items']:
				if 'project_slug' in build and 'name' in build and 'status' in build:
					result.append(normalize_data(build['project_slug'] + '/' + build['name'], build['status']))
	return result


def fetch_pipeline_ids(host : str, organization : str, slug : str, token : str) -> List[str]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/api/v2/project/' + slug + '/pipeline', headers = create_headers(token))
	elif host and organization and token:
		response = request.get(host + '/api/v2/pipeline?org-slug=' + organization, headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'items' in data:
			pipeline = helper.get_first(data['items'])

			if pipeline and 'id' in pipeline:
				result.append(pipeline['id'])
	return result


def create_headers(token : str) -> Headers:
	return \
	{
		'Accept': 'application/json',
		'Circle-Token': token
	}



import copy
from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

LIGHTS = None


def get_lights() -> Any:
	global LIGHTS

	if not LIGHTS:
		LIGHTS = get_api().get_lights()
	return LIGHTS


def filter_lights(lights : Any, light_ips : List[str]) -> Any:
	if light_ips:
		for light in copy.copy(lights):
			if light.get_ip_addr() not in light_ips:
				lights.remove(light)
	return lights


def process_lights(lights : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process lights

	for light in lights:
		set_light(light, color.get_by_status(status))
		result.append(
		{
			'name': 'lifx_light',
			'type': 'light',
			'description': helper.create_description(light.get_label(), light.get_ip_addr()),
			'status': status
		})
	return result


def set_light(light : Any, color_config : Color) -> None:
	light.set_power('on')
	light.set_color(
	[
		color_config['hue'],
		color_config['saturation'][2],
		color_config['brightness'][2],
		color_config['kelvin']
	])



from chroma_feedback import producer

METADATA =\
{
	'name': 'chroma-feedback',
	'description': 'Turn your RGB powered hardware into a status indicator for continuous integration, continuous deployment and infrastructure monitoring',
	'version': '12.3.0',
	'license': 'MIT',
	'keywords': ' '.join(producer.__all__),
	'author': 'Henry Ruhs',
	'author_email': 'info@henryruhs.com',
	'url': 'https://chroma-feedback.com'
}


def get(key : str) -> str:
	return METADATA[key]



import ctypes
import os
import platform
import sys
from typing import Any, Dict


def parse_slug(slug : str) -> Dict[str, str]:
	if slug:
		return dict(zip(['workspace', 'project'], slug.split('/')))
	return {}


def create_description(name : str, selector : str) -> str:
	if name and selector:
		return name + ' [' + selector + ']'
	else:
		return name or selector


def is_root() -> bool:
	try:
		return os.getuid() == 0 #type:ignore
	except AttributeError:
		try:
			return ctypes.windll.shell32.IsUserAnAdmin() == 1 #type:ignore
		except AttributeError:
			return False


def is_linux() -> bool:
	return to_lower_case(platform.system()) == 'linux'


def is_mac() -> bool:
	return to_lower_case(platform.system()) == 'darwin'


def is_windows() -> bool:
	return to_lower_case(platform.system()) == 'windows'


def to_lower_case(__string__ : Any) -> str:
	return str(__string__).lower()


def has_argument(argument : str) -> bool:
	return any(argv.startswith(argument) for argv in sys.argv)


def get_first(__list__ : Any) -> Any:
	return next(iter(__list__), None)


def get_last(__list__ : Any) -> Any:
	return next(reversed(__list__), None)


def remove_duplicate(__list__ : Any) -> Any:
	return list(dict.fromkeys(__list__))



import copy
from typing import Any, List

from chroma_feedback import color, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

GROUPS = None


def get_groups(bridge_ip : str) -> Any:
	global GROUPS

	if not GROUPS:
		GROUPS = get_api(bridge_ip).get_group()
	return GROUPS


def filter_groups(groups : Any, group_names : List[str]) -> Any:
	if group_names:
		for index in copy.copy(groups):
			if groups[index]['name'] not in group_names:
				del groups[index]
	return groups


def process_groups(groups : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process groups

	for index in groups:
		set_group(groups[index]['name'], color.get_by_status(status))
		result.append(
		{
			'name': 'philips_hue',
			'type': 'group',
			'description': groups[index]['name'],
			'status': status
		})
	return result


def set_group(group_name : str, color_config : Color) -> None:
	get_api(None).set_group(group_name,
	{
		'hue': color_config['hue'],
		'sat': color_config['saturation'][1],
		'bri': color_config['brightness'][1],
		'on': True,
		'alert': 'none'
	})



from .core import process

__all__ =\
[
	'agile_innovative_blinkstick',
	'elgato_streamdeck',
	'embrava_blynclight',
	'kuando_busylight',
	'lifx_light',
	'luxafor_flag',
	'magic_hue',
	'nanoleaf_light',
	'philips_hue',
	'razer_chroma',
	'thingm_blink1',
	'wiz_light',
	'xiaomi_yeelight'
]



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data, normalize_slug

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--bamboo-host', required = True)
		program.add_argument('--bamboo-slug', action = 'append', required = True)
		program.add_argument('--bamboo-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.bamboo_slug:
		result.extend(fetch(ARGS.bamboo_host, slug, ARGS.bamboo_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		slug = normalize_slug(slug)
		response = request.get(host + '/rest/api/latest/result/' + slug, headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'results' in data and 'result' in data['results']:
			for project in data['results']['result']:
				if 'key' in project and 'buildState' in project:
					result.append(normalize_data(project['key'], project['buildState']))
		elif 'key' in data and 'buildState' in data:
			result.append(normalize_data(data['key'], data['buildState']))
	return result



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--buddy-host', default = 'https://api.buddy.works')
		program.add_argument('--buddy-slug', action = 'append', required = True)
		program.add_argument('--buddy-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.buddy_slug:
		slug_fragment = helper.parse_slug(slug)

		if 'workspace' in slug_fragment:
			if 'project' in slug_fragment:
				result.extend(fetch(ARGS.buddy_host, slug_fragment['workspace'], slug_fragment['project'], ARGS.buddy_token))
			else:
				project_names = fetch_project_names(ARGS.buddy_host, slug_fragment['workspace'], ARGS.buddy_token)

				if project_names:
					for project_name in project_names:
						result.extend(fetch(ARGS.buddy_host, slug_fragment['workspace'], project_name, ARGS.buddy_token))
	return result


def fetch(host: str, workspace: str, project : str, token: str) -> List[Producer]:
	result = []
	response = None

	if host and workspace and project and token:
		response = request.get(host + '/workspaces/' + workspace + '/projects/' + project + '/pipelines/', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'pipelines' in data:
			pipeline = helper.get_first(data['pipelines'])

			if pipeline and 'last_execution_status' in pipeline:
				result.append(normalize_data(workspace + '/' + project, pipeline['last_execution_status']))
	return result


def fetch_project_names(host : str, workspace : str, token : str) -> List[str]:
	result = []
	response = None

	if host and workspace and token:
		response = request.get(host + '/workspaces/' + workspace + '/projects', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'projects' in data:
			for project in data['projects']:
				if 'name' in project:
					result.append(project['name'])
	return result




import os
from setuptools.command.install import install

from chroma_feedback import helper


class InstallCommand(install):
	@staticmethod
	def write_rule() -> None:
		with open('/etc/udev/rules.d/99-chroma-feedback.rules', 'w+', encoding = 'utf8') as file:
			file.write('SUBSYSTEM=="usb", MODE="0666"')
			file.close()


	@staticmethod
	def load_rule() -> None:
		os.system('udevadm control --reload')
		os.system('udevadm trigger')


	def run(self) -> None:
		install.run(self)

		if helper.is_linux() is True and helper.is_root() is True:
			self.write_rule()
			self.load_rule()



import sys
from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, logger, ssdp, wording
from chroma_feedback.typing import Consumer, ProducerReport
from .light import get_lights, process_lights

ARGS = None


def support() -> bool:
	return helper.is_linux() is True or helper.is_mac() is True or helper.is_windows() is True


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		light_ips = None

		if not helper.has_argument('--nanoleaf-light-light-ip'):
			light_ips = discover_light_ips()
		if light_ips:
			program.add_argument('--nanoleaf-light-light-ip', default = light_ips)
		else:
			program.add_argument('--nanoleaf-light-light-ip', action = 'append', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run(producer_report : List[ProducerReport]) -> List[Consumer]:
	lights = get_lights(ARGS.nanoleaf_light_light_ip)

	if not lights:
		logger.error(wording.get('light_not_found') + wording.get('exclamation_mark'))
		sys.exit()
	return process_lights(lights, producer_report)


def discover_light_ips() -> List[str]:
	light_ips = ssdp.discover_ips('239.255.255.250', 1900, 'nanoleaf:nl29')

	if light_ips is None:
		logger.warn(wording.get('ip_not_found').format('light') + wording.get('exclamation_mark'))
	return light_ips



import sys

from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QApplication

APPLICATION = None
TIMER = None


def get_application() -> QApplication:
	global APPLICATION

	if APPLICATION is None:
		APPLICATION = QApplication(sys.argv)
	return APPLICATION


def get_timer() -> QTimer:
	global TIMER

	if TIMER is None:
		TIMER = QTimer()
	return TIMER


def is_created() -> bool:
	global APPLICATION, TIMER

	return APPLICATION is not None and TIMER is not None



import copy
from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

DEVICES = None


def get_devices() -> Any:
	global DEVICES

	if not DEVICES:
		DEVICES = get_api().devices
	return DEVICES


def filter_devices(devices : Any, device_serials : List[str]) -> Any:
	if device_serials:
		for device in copy.copy(devices):
			if device.serial not in device_serials:
				devices.remove(device)
	return devices


def process_devices(devices : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process devices

	for device in devices:
		if set_device(device, color.get_by_status(status)):
			result.append(
			{
				'name': 'razer_chroma',
				'type': 'device',
				'description': helper.create_description(device.name, device.serial),
				'status': status
			})
	return result


def set_device(device : Any, color_config : Color) -> bool:
	if device.has('brightness'):
		device.brightness = color_config['brightness'][0]
	if device.fx.has('static'):
		return device.fx.static(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2])
	return use_effect(device, color_config, 'static')


def use_effect(device : Any, color_config : Color, effect_name : str) -> bool:
	effect_state = False
	parts =\
	{
		'logo': 'logo',
		'scroll': 'scroll_wheel',
		'left': 'left',
		'right': 'right',
		'backlight': 'backlight'
	}

	for part_key, part_value in parts.items():
		if device.fx.has(part_key + '_' + effect_name):
			effect_function = getattr(getattr(device.fx.misc, part_value), effect_name)
			effect_state = effect_function(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2]) or effect_state
	return effect_state



import base64
from time import sleep
from typing import Any

import requests
from requests import RequestException, Response

from chroma_feedback import logger, wording
from chroma_feedback.typing import Headers


def get(url : str, headers : Headers = None) -> Response:
	try:
		response = requests.get(url, headers = headers, timeout = 10)
		logger.debug(wording.get('fetch_request').format('GET', response.url, response.status_code))
		return response
	except RequestException:
		logger.warn(wording.get('retry_request').format('GET', 10))
		sleep(10)
		return get(url, headers)


def post(url : str, data : Any = None, headers : Headers = None) -> Response:
	try:
		response = requests.post(url, data, headers = headers, timeout = 10)
		logger.debug(wording.get('fetch_request').format('POST', response.url, response.status_code))
		return response
	except RequestException:
		logger.warn(wording.get('retry_request').format('POST', 10))
		sleep(10)
		return post(url, data, headers)


def parse_json(response: Response) -> Any:
	try:
		return response.json()
	except ValueError:
		return None


def create_basic_auth_headers(username : str, password : str) -> Headers:
	username_password = username + ':' + password

	return\
	{
		'Accept': 'application/json',
		'Authorization': 'Basic ' + base64.b64encode(username_password.encode('utf-8')).decode('ascii')
	}


def create_bearer_auth_headers(token : str) -> Headers:
	return\
	{
		'Accept': 'application/json',
		'Authorization': 'Bearer ' + token
	}



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Headers, Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--github-host', default = 'https://api.github.com')
		program.add_argument('--github-slug', action = 'append', required = True)
		program.add_argument('--github-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.github_slug:
		slug_fragment = helper.parse_slug(slug)

		if 'workspace' in slug_fragment:
			if 'project' in slug_fragment:
				result.extend(fetch(ARGS.github_host, slug_fragment['workspace'] + '/' + slug_fragment['project'], ARGS.github_token))
			else:
				repository_names = fetch_repository_names(ARGS.github_host, slug_fragment['workspace'], ARGS.github_token)

				if repository_names:
					for repository_name in repository_names:
						result.extend(fetch(ARGS.github_host, repository_name, ARGS.github_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/repos/' + slug + '/actions/runs', headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'workflow_runs' in data:
			build = helper.get_first(data['workflow_runs'])

			if build and 'repository' in build and 'full_name' in build['repository'] and 'html_url' in build and 'status' in build and 'conclusion' in build:
				result.append(normalize_data(build['repository']['full_name'], build['html_url'], build['status'], build['conclusion']))
	return result


def fetch_repository_names(host : str, username : str, token : str) -> List[str]:
	result = []
	response = None

	if host and username and token:
		response = request.get(host + '/search/repositories?q=user:' + username, headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'items' in data:
			for repository in data['items']:
				if 'full_name' in repository:
					result.append(repository['full_name'])
	return result


def create_headers(token : str) -> Headers:
	return\
	{
		'Accept': 'application/vnd.github.v3+json',
		'Authorization': 'Token ' + token
	}



from typing import Any, List

from chroma_feedback import color, helper, logger, metadata, wording
from chroma_feedback.typing import Consumer, ConsumerReport, Producer, ProducerReport, Status


def create_producer_report(producer_result : List[Producer]) -> List[ProducerReport]:
	report : List[ProducerReport] = []

	# process result

	for result in producer_result:
		if result['status'] == 'passed':
			report.append(
			{
				'name': result['name'],
				'symbol': wording.get('tick'),
				'message': wording.get('has_status').format(result['slug'], result['name'], 'passed'),
				'url': result['url'],
				'status': result['status']
			})
		if result['status'] == 'started':
			report.append(
			{
				'name': result['name'],
				'symbol': wording.get('hourglass'),
				'message': wording.get('has_status').format(result['slug'], result['name'], 'started'),
				'url': result['url'],
				'status': result['status']
			})
		if result['status'] in ['errored', 'warned', 'failed']:
			report.append(
			{
				'name': result['name'],
				'symbol': wording.get('cross'),
				'message': wording.get('has_status').format(result['slug'], result['name'], result['status']),
				'url': result['url'],
				'status': result['status']
			})
	return report


def create_consumer_report(consumer_result : List[Consumer]) -> List[ConsumerReport]:
	report : List[ConsumerReport] = []

	# process result

	for result in consumer_result:
		if result['status'] == 'passed':
			report.append(
			{
				'name': result['name'],
				'symbol': wording.get('tick'),
				'message': wording.get('set_status').format(result['description'], result['status']),
				'status': result['status']
			})
		if result['status'] == 'started':
			report.append(
			{
				'name': result['name'],
				'symbol': wording.get('hourglass'),
				'message': wording.get('set_status').format(result['description'], result['status']),
				'status': result['status']
			})
		if result['status'] in ['errored', 'warned', 'failed']:
			report.append(
			{
				'name': result['name'],
				'symbol': wording.get('cross'),
				'message': wording.get('set_status').format(result['description'], result['status']),
				'status': result['status']
			})
	return report


def resolve_report_status(producer_report : List[ProducerReport]) -> Status:
	status : Status = 'passed'

	# process report

	for report in producer_report:
		if report['status'] == 'started' and status not in ['errored', 'warned', 'failed']:
			status = 'started'
		if report['status'] == 'errored' and status not in ['warned', 'failed']:
			status = 'errored'
		if report['status'] == 'warned' and status != 'failed':
			status = 'warned'
		if report['status'] == 'failed':
			status = 'failed'
	return status


def print_header() -> None:
	logger.info(metadata.get('name') + ' ' + metadata.get('version') + ' ' + wording.get('by') + ' ' + metadata.get('author'))
	logger.info()


def print_report(report : List[Any]) -> None:
	for value in report:
		if helper.is_windows() is False:
			logger.info(color.format_by_status(value['symbol'], value['status']) + ' ' + value['message'])
		else:
			logger.info(value['message'])
	logger.info()



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Headers, Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--datadog-host', default = 'https://api.datadoghq.com')
		program.add_argument('--datadog-slug', action = 'append', required = True)
		program.add_argument('--datadog-api-key', required = True)
		program.add_argument('--datadog-application-key', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.datadog_slug:
		result.extend(fetch(ARGS.datadog_host, slug, ARGS.datadog_api_key, ARGS.datadog_application_key))
	return result


def fetch(host : str, slug : str, api_key : str, application_key : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and api_key and application_key:
		response = request.get(host + '/api/v1/monitor/' + slug, headers = create_headers(api_key, application_key))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'name' in data and 'overall_state' in data:
			result.append(normalize_data(data['name'], data['overall_state']))
	return result


def create_headers(api_key : str, application_key : str) -> Headers:
	return \
	{
		'Accept': 'application/json',
		'DD-API-KEY': api_key,
		'DD-APPLICATION-KEY': application_key
	}



from typing import Any, List

from chroma_feedback import color, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

GROUPS : List[Any] = []


def get_groups(group_names : List[str]) -> List[Any]:
	global GROUPS

	if not GROUPS:
		if group_names:
			for group_name in group_names:
				group = get_api().get_devices_by_group(group_name)

				if group.get_device_list():
					GROUPS.append(group)
	return GROUPS


def get_group_name(group : Any) -> Any:
	for device in group.get_device_list():
		return device.get_group_label()


def process_groups(groups : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process groups

	for group in groups:
		set_group(group, color.get_by_status(status))
		result.append(
		{
			'name': 'lifx_light',
			'type': 'group',
			'description': get_group_name(group),
			'status': status
		})
	return result


def set_group(group : Any, color_config : Color) -> None:
	group.set_power('on')
	group.set_color(
	[
		color_config['hue'],
		color_config['saturation'][2],
		color_config['brightness'][2],
		color_config['kelvin']
	])



import sys
import webbrowser
from typing import List

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QBrush, QColor, QIcon, QPainter, QPixmap
from PyQt5.QtWidgets import QAction, QMenu, QSystemTrayIcon

from chroma_feedback import color, logger, loop, reporter, wording
from chroma_feedback.typing import ProducerReport, Status

SYSTRAY = None
MENU = None


def create(producer_report : List[ProducerReport]) -> None:
	global SYSTRAY, MENU

	if not SYSTRAY:
		SYSTRAY = QSystemTrayIcon()
	if not MENU:
		MENU = QMenu()
	update(producer_report)
	SYSTRAY.show()


def update(producer_report : List[ProducerReport]) -> None:
	global SYSTRAY, MENU

	status = reporter.resolve_report_status(producer_report)

	update_menu(producer_report)
	SYSTRAY.setContextMenu(MENU)
	SYSTRAY.setIcon(create_icon(status))


def is_created() -> bool:
	global SYSTRAY, MENU

	return SYSTRAY is not None and MENU is not None


def update_menu(producer_report : List[ProducerReport]) -> None:
	global MENU

	MENU.clear()

	# process report

	for report in producer_report:
		item_report = MENU.addAction(report['message'])
		item_report.setIcon(create_icon(report['status']))
		item_report.setIconVisibleInMenu(True)
		if 'url' in report and report['url']:
			item_report.triggered.connect(lambda __, url = report['url'] : webbrowser.open(url))
		else:
			item_report.setDisabled(True)
	if producer_report:
		MENU.addSeparator()

	# handle action

	item_start = MENU.addAction(wording.get('start'))
	item_stop = MENU.addAction(wording.get('stop'))
	item_exit = MENU.addAction(wording.get('exit'))
	item_start.setVisible(loop.get_timer().isActive() is False)
	item_stop.setVisible(loop.get_timer().isActive() is True)
	item_start.triggered.connect(lambda : action_start(item_start, item_stop))
	item_stop.triggered.connect(lambda : action_stop(item_start, item_stop))
	item_exit.triggered.connect(action_exit)


def create_icon(status : Status) -> QIcon:
	color_config = color.get_by_status(status)
	pixmap = QPixmap(100, 100)
	pixmap.fill(Qt.transparent)
	painter = QPainter(pixmap)
	painter.setBrush(QBrush(QColor(color_config['rgb'][0], color_config['rgb'][1], color_config['rgb'][2]), Qt.SolidPattern))
	painter.drawEllipse(20, 20, 60, 60)
	painter.end()
	return QIcon(pixmap)


def action_start(item_start : QAction, item_stop : QAction) -> None:
	item_start.setVisible(False)
	item_stop.setVisible(True)
	loop.get_timer().start()


def action_stop(item_start : QAction, item_stop : QAction) -> None:
	item_start.setVisible(True)
	item_stop.setVisible(False)
	loop.get_timer().stop()


def action_exit() -> None:
	logger.info(wording.get('goodbye') + wording.get('exclamation_mark'))
	sys.exit()



import os
import socket
from typing import List

from chroma_feedback import helper


def discover_ips(host : str, port : int, search_target : str = 'ssdp:all') -> List[str]:
	ips = []
	message =\
	[
		'M-SEARCH * HTTP/1.1',
		'MAN: "ssdp:discover"',
		'ST: ' + search_target
	]

	# discover ips

	discovery = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
	discovery.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
	discovery.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, 1)
	discovery.settimeout(2)
	discovery.sendto(os.linesep.join(message).encode(), (host, port))

	# receive ips

	while True:
		try:
			ips.append(discovery.recvfrom(65507)[1][0])
		except socket.timeout:
			break
	return helper.remove_duplicate(ips)



from chroma_feedback import helper
from chroma_feedback.typing import Color, Status

COLOR =\
{
	'red': '\033[0;31m',
	'green': '\033[0;32m',
	'yellow': '\033[0;33m',
	'blue': '\033[0;34m',
	'white': '\033[0;37m',
	'end': '\033[0m'
}

def format_by_status(text : str, status : Status) -> str:
	if status == 'started':
		return format_started(text)
	if status == 'errored':
		return format_errored(text)
	if status == 'warned':
		return format_warned(text)
	if status == 'failed':
		return format_failed(text)
	return format_passed(text)


def format_passed(text : str) -> str:
	if helper.is_windows() is False:
		return COLOR['green'] + text + COLOR['end']
	return text


def format_started(text : str) -> str:
	if helper.is_windows() is False:
		return COLOR['blue'] + text + COLOR['end']
	return text


def format_errored(text : str) -> str:
	if helper.is_windows() is False:
		return COLOR['white'] + text + COLOR['end']
	return text


def format_warned(text : str) -> str:
	if helper.is_windows() is False:
		return COLOR['yellow'] + text + COLOR['end']
	return text


def format_failed(text : str) -> str:
	if helper.is_windows() is False:
		return COLOR['red'] + text + COLOR['end']
	return text


def get_by_status(status : Status) -> Color:
	if status == 'started':
		return get_started()
	if status == 'errored':
		return get_errored()
	if status == 'warned':
		return get_warned()
	if status == 'failed':
		return get_failed()
	return get_passed()


def get_passed() -> Color:
	return\
	{
		'name': 'green',
		'rgb':
		[
			0,
			255,
			0
		],
		'hue': 26000,
		'saturation':
		[
			100,
			255,
			65535
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}


def get_started() -> Color:
	return\
	{
		'name': 'blue',
		'rgb':
		[
			0,
			0,
			255
		],
		'hue': 46000,
		'saturation':
		[
			100,
			255,
			65535
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}


def get_errored() -> Color:
	return\
	{
		'name': 'white',
		'rgb':
		[
			255,
			255,
			255
		],
		'hue': 58000,
		'saturation':
		[
			0,
			0,
			0
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}


def get_warned() -> Color:
	return\
	{
		'name': 'yellow',
		'rgb':
		[
			255,
			255,
			0
		],
		'hue': 10000,
		'saturation':
		[
			100,
			255,
			65535
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}


def get_failed() -> Color:
	return\
	{
		'name': 'red',
		'rgb':
		[
			255,
			0,
			0
		],
		'hue': 0,
		'saturation':
		[
			100,
			255,
			65535
		],
		'brightness':
		[
			100,
			255,
			65535
		],
		'kelvin': 3500
	}



from argparse import ArgumentParser
from typing import Any, List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--codeship-host', default = 'https://api.codeship.com')
		program.add_argument('--codeship-slug', action = 'append')
		program.add_argument('--codeship-username', required = True)
		program.add_argument('--codeship-password', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []
	authentication = fetch_authentication(ARGS.codeship_host, ARGS.codeship_username, ARGS.codeship_password)

	if 'organizations' in authentication and 'token' in authentication:
		for organization in authentication['organizations']:
			projects = fetch_projects(ARGS.codeship_host, organization['uuid'], authentication['token'])

			if projects:
				for project in projects:
					if ARGS.codeship_slug is None or project['name'] in ARGS.codeship_slug:
						result.extend(fetch(ARGS.codeship_host, organization['uuid'], project['name'], project['uuid'], authentication['token']))
	return result


def fetch(host : str, organization_id : str, project_name : str, project_id : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and organization_id and project_id and token:
		response = request.get(host + '/v2/organizations/' + organization_id + '/projects/' + project_id + '/builds', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'builds' in data:
			build = helper.get_first(data['builds'])

			if build and 'status' in build:
				result.append(normalize_data(project_name, build['status']))
	return result


def fetch_authentication(host : str, username : str, password : str) -> Any:
	result = {}
	response = None

	if host and username and password:
		response = request.post(host + '/v2/auth', headers = request.create_basic_auth_headers(username, password))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'access_token' in data and 'organizations' in data:
			result['token'] = data['access_token']
			result['organizations'] = data['organizations']
	return result


def fetch_projects(host : str, organization_id : str, token : str) -> List[Any]:
	result = []
	response = None

	if host and organization_id and token:
		response = request.get(host + '/v2/organizations/' + organization_id + '/projects', headers = request.create_bearer_auth_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'projects' in data:
			for project in data['projects']:
				result.append(project)
	return result



from chroma_feedback import helper
from chroma_feedback.typing import Producer, Status


def normalize_data(slug : str, status : str) -> Producer:
	return\
	{
		'name': 'datadog',
		'slug': slug,
		'url': None,
		'status': normalize_status(status)
	}


def normalize_status(status : str) -> Status:
	status = helper.to_lower_case(status).replace(' ', '_')

	if status == 'skipped':
		return 'skipped'
	if status == 'no_data':
		return 'errored'
	if status == 'warn':
		return 'warned'
	if status == 'alert':
		return 'failed'
	return 'passed'



#!/usr/bin/env python3

from setuptools import setup
from chroma_feedback import metadata
from chroma_feedback.install import InstallCommand

setup(
	name = metadata.get('name'),
	description = metadata.get('description'),
	long_description = open('README.md').read(),
	long_description_content_type = 'text/markdown',
	version = metadata.get('version'),
	license = metadata.get('license'),
	keywords = metadata.get('keywords'),
	author = metadata.get('author'),
	author_email = metadata.get('author_email'),
	url = metadata.get('url'),
	packages =
	[
		'chroma_feedback',
		'chroma_feedback.producer',
		'chroma_feedback.producer.appveyor',
		'chroma_feedback.producer.azure',
		'chroma_feedback.producer.bamboo',
		'chroma_feedback.producer.bitbucket',
		'chroma_feedback.producer.buddy',
		'chroma_feedback.producer.circle',
		'chroma_feedback.producer.codeship',
		'chroma_feedback.producer.custom',
		'chroma_feedback.producer.datadog',
		'chroma_feedback.producer.github',
		'chroma_feedback.producer.gitlab',
		'chroma_feedback.producer.heroku',
		'chroma_feedback.producer.jenkins',
		'chroma_feedback.producer.netlify',
		'chroma_feedback.producer.teamcity',
		'chroma_feedback.producer.travis',
		'chroma_feedback.producer.vercel',
		'chroma_feedback.consumer',
		'chroma_feedback.consumer.agile_innovative_blinkstick',
		'chroma_feedback.consumer.elgato_streamdeck',
		'chroma_feedback.consumer.embrava_blynclight',
		'chroma_feedback.consumer.kuando_busylight',
		'chroma_feedback.consumer.lifx_light',
		'chroma_feedback.consumer.luxafor_flag',
		'chroma_feedback.consumer.magic_hue',
		'chroma_feedback.consumer.nanoleaf_light',
		'chroma_feedback.consumer.philips_hue',
		'chroma_feedback.consumer.razer_chroma',
		'chroma_feedback.consumer.thingm_blink1',
		'chroma_feedback.consumer.wiz_light',
		'chroma_feedback.consumer.xiaomi_yeelight'
	],
	cmdclass =
	{
		'install': InstallCommand
	},
	entry_points =
	{
		'console_scripts':
		[
			'chroma-feedback = chroma_feedback.core:cli',
		]
	},
	install_requires =
	[
		'busylight-for-humans==0.21.1',
		'lifxlan==1.2.7',
		'nanoleafapi==2.1.1',
		'phue==1.1',
		'python-magichue==0.3.2',
		'pyqt5==5.15.7',
		'pywizlight==0.5.14',
		'requests==2.28.1',
		'streamdeck==0.9.1',
		'yeelight==0.7.10'
	],
	tests_require =
	[
		'flake8==5.0.0',
		'mock==4.0.3',
		'mypy==0.971',
		'pyqt5-stubs==5.15.6.0',
		'pytest==7.1.2',
		'pytest-cov==3.0.0',
		'pytest-mock==3.8.2',
		'types-mock==4.0.15',
		'types-requests==2.28.6'
	]
)



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--jenkins-host', required = True)
		program.add_argument('--jenkins-slug', action = 'append', required = True)
		program.add_argument('--jenkins-username', required = True)
		program.add_argument('--jenkins-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	for slug in ARGS.jenkins_slug:
		result.extend(fetch(ARGS.jenkins_host, slug, ARGS.jenkins_username, ARGS.jenkins_token))
	return result


def fetch(host : str, slug : str, username : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and username and token:
		response = request.get(host + '/job/' + slug + '/lastBuild/api/json', headers = request.create_basic_auth_headers(username, token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'result' in data and 'building' in data:
			result.append(normalize_data(slug, data['result'], data['building']))
	return result



from chroma_feedback import helper
from chroma_feedback.typing import Producer, Status


def normalize_data(slug : str, url : str, status : str, is_paused : bool, is_running : bool) -> Producer:
	return\
	{
		'name': 'teamcity',
		'slug': slug,
		'url': url,
		'status': normalize_status(status, is_paused, is_running)
	}


def normalize_status(status : str, is_paused : bool, is_running : bool) -> Status:
	status = helper.to_lower_case(status)

	if is_paused is True:
		return 'skipped'
	if is_running is True:
		return 'started'
	if status == 'error':
		return 'errored'
	if status == 'failure':
		return 'failed'
	return 'passed'



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Headers, Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--heroku-host', default = 'https://api.heroku.com')
		program.add_argument('--heroku-slug', action = 'append')
		program.add_argument('--heroku-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.heroku_slug:
		for slug in ARGS.heroku_slug:
			result.extend(fetch(ARGS.heroku_host, slug, ARGS.heroku_token))
	else:
		application_ids = fetch_application_ids(ARGS.heroku_host, ARGS.heroku_token)

		if application_ids:
			for application_id in application_ids:
				result.extend(fetch(ARGS.heroku_host, application_id, ARGS.heroku_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/apps/' + slug + '/releases', headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)
		build = helper.get_last(data)

		if build and 'app' in build and 'name' in build['app'] and 'status' in build:
			result.append(normalize_data(build['app']['name'], build['status']))
	return result


def fetch_application_ids(host : str, token : str) -> List[str]:
	result = []
	response = None

	if host and token:
		response = request.get(host + '/apps', headers = create_headers(token))

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		for application in data:
			if 'id' in application:
				result.append(application['id'])
	return result


def create_headers(token : str) -> Headers:
	return\
	{
		'Accept': 'application/vnd.heroku+json;version=3',
		'Authorization': 'Bearer ' + token
	}



import copy
from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

LIGHTS = None


def get_lights(bridge_ip : str) -> Any:
	global LIGHTS

	if not LIGHTS:
		LIGHTS = get_api(bridge_ip).get_light_objects()
	return LIGHTS


def filter_lights(lights : Any, light_ids : List[str]) -> Any:
	if light_ids:
		for light in copy.copy(lights):
			if light._get('uniqueid') not in light_ids:
				lights.remove(light)
	return lights


def process_lights(lights : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	# process lights

	for light in lights:
		set_light(light.name, color.get_by_status(status))
		result.append(
		{
			'name': 'philips_hue',
			'type': 'light',
			'description': helper.create_description(light.name, light._get('uniqueid')),
			'status': status
		})
	return result


def set_light(light_name : str, color_config : Color) -> None:
	get_api(None).set_light(light_name,
	{
		'hue': color_config['hue'],
		'sat': color_config['saturation'][1],
		'bri': color_config['brightness'][1],
		'on': True,
		'alert': 'none'
	})
